@@ -22,7 +22,7 @@ class RocketDepot:
     def __init__(self):
         # Default options.  Overridden by config file.
         self.options = {
-            'host': 'host.example.com',
+            'host': '',
             # set the default RDP user to the local user by default
             'user': os.environ.get('USER', 'user'),
             'geometry': '1024x768',
@@ -40,8 +40,9 @@ class RocketDepot:
         self.configfile = '%s/.config/rocket-depot/config.ini' % self.homedir
         self.config = ConfigParser.RawConfigParser()
         self.config.read(self.configfile)
-        self.read_config('defaults')
-        self.save_config('defaults')
+        self.read_config('DEFAULT')
+        self.save_config('DEFAULT')
+        self.saved_hosts = self.list_profiles()
         self.mw = MainWindow(self)
 
     # Create config dir
@@ -59,37 +60,47 @@ class RocketDepot:
             self.config.write(f)
 
     # Save options to the config file
-    def save_config(self, section):
-        # add the new section if it doesn't exist
-        if not self.config.has_section(section):
-            self.config.add_section(section)
+    def save_config(self, host):
+    # add the new section if it doesn't exist
+        if (not self.config.has_section(host) and
+                host != 'DEFAULT' and host != ''):
+            self.config.add_section(host)
         # Set all selected options
         for opt in self.options:
-            self.config.set(section, opt, self.options[opt])
+            if opt != 'host':
+                self.config.set(host, opt, self.options[opt])
         self.write_config()
+        self.saved_hosts = self.list_profiles()
+
 
     # Delete a section from the config file
-    def delete_config(self, section):
-        self.config.remove_section(section)
+    def delete_config(self, host):
+        self.config.remove_section(host)
         self.write_config()
+        self.saved_hosts = self.list_profiles()
+
 
     # Set options based on section in config file
-    def read_config(self, section):
+    def read_config(self, host):
         if os.path.exists(self.configfile):
             for opt in self.options:
-                if not self.config.has_option(section, opt):
+                if not self.config.has_option(host, opt):
                     self.options[opt] = ''
                 else:
-                    self.options[opt] = self.config.get(section, opt)
+                    self.options[opt] = self.config.get(host, opt)
+            if host == 'DEFAULT':
+                self.options['host'] = ''
+            else:
+                self.options['host'] = host
+
 
     # Make a list of all profiles in config file.  Sort the order
-    # alphabetically, except special 'defaults' profile always comes first
+    # alphabetically, except special 'DEFAULT' profile always comes first
     def list_profiles(self):
         profiles_list = sorted(self.config.sections())
-        defaults_index = profiles_list.index('defaults')
-        profiles_list.insert(0, profiles_list.pop(defaults_index))
         return profiles_list
 
+
     # Check for given host in freerdp's known_hosts file before connecting
     def check_known_hosts(self, host):
         known_hosts = '%s/.config/freerdp/known_hosts' % self.homedir
@@ -254,29 +265,35 @@ class MainWindow(Gtk.Window):
         self.add(grid)
 
         # Labels for text entry fields and comboboxes
-        profileslabel = Gtk.Label(label="Profile")
         hostlabel = Gtk.Label(label="Host")
         userlabel = Gtk.Label(label="Username")
         geometrylabel = Gtk.Label(label="Geometry")
         clioptionslabel = Gtk.Label(label="CLI Options")
         programlabel = Gtk.Label(label="RDP Client")
 
-        # Profiles combobox
-        self.profiles_combo = Gtk.ComboBoxText.new_with_entry()
-        self.profiles_combo.set_tooltip_text('List of saved connection '
-                                             'profiles')
-        self.populate_profiles_combobox()
-        self.profiles_combo.connect("changed", self.on_profiles_combo_changed)
-        # If an existing profile name has been typed into the profiles
+        # Host combobox
+        self.host_combo_store = Gtk.ListStore(str)
+        self.populate_host_combobox()
+        self.host_combo = Gtk.ComboBox.new_with_model_and_entry(self.host_combo_store)
+        self.host_combo.connect("changed", self.on_host_combo_changed)
+        self.host_combo.set_entry_text_column(0)
+        self.host_combo.set_tooltip_text('Enter hostname/IP or list saved hosts')
+        self.host_entry = self.host_combo.get_child()
+        # Auto-complete
+        completion = Gtk.EntryCompletion()
+        completion.set_model(self.host_combo_store)
+        self.host_entry.set_completion(completion)
+        completion.set_text_column(0)
+        #completion.connect('match-selected', self.match_cb)
+        completion.set_inline_completion(True)
+        #entry.connect('activate', self.activate_cb)
+        # If an existing profile name has been typed into the host
         # combobox, allow the 'enter' key to launch the RDP client
-        profiles_combo_entry = self.profiles_combo.get_children()[0]
-        profiles_combo_entry.connect("activate", self.enter_connect,
-                                     profiles_combo_entry)
+        host_combo_entry = self.host_combo.get_children()[0]
+        host_combo_entry.connect("activate", self.enter_connect,
+                                     host_combo_entry)
 
         # Text entry fields
-        self.hostentry = Gtk.Entry()
-        self.hostentry.set_tooltip_text('Hostname or IP address of RDP server')
-        self.hostentry.connect("activate", self.enter_connect, self.hostentry)
         self.userentry = Gtk.Entry()
         self.userentry.set_tooltip_text('''RDP username.
 Domain credentials may be entered in domain\username format:
@@ -341,17 +358,14 @@ Useful for diagnosing connection problems''')
 
         # Grid to which we attach all of our widgets
         grid.attach(menubar, 0, 0, 12, 4)
-        grid.attach(profileslabel, 0, 4, 4, 4)
-        grid.attach(hostlabel, 0, 8, 4, 4)
-        grid.attach(userlabel, 0, 12, 4, 4)
-        grid.attach(geometrylabel, 0, 16, 4, 4)
-        grid.attach(clioptionslabel, 0, 20, 4, 4)
-        grid.attach(programlabel, 0, 24, 4, 4)
-        grid.attach(self.homedirbutton, 0, 28, 4, 4)
-        grid.attach(self.terminalbutton, 0, 32, 4, 4)
-        grid.attach_next_to(self.profiles_combo, profileslabel,
-                            Gtk.PositionType.RIGHT, 8, 4)
-        grid.attach_next_to(self.hostentry, hostlabel,
+        grid.attach(hostlabel, 0, 4, 4, 4)
+        grid.attach(userlabel, 0, 8, 4, 4)
+        grid.attach(geometrylabel, 0, 12, 4, 4)
+        grid.attach(clioptionslabel, 0, 16, 4, 4)
+        grid.attach(programlabel, 0, 20, 4, 4)
+        grid.attach(self.homedirbutton, 0, 24, 4, 4)
+        grid.attach(self.terminalbutton, 0, 28, 4, 4)
+        grid.attach_next_to(self.host_combo, hostlabel,
                             Gtk.PositionType.RIGHT, 8, 4)
         grid.attach_next_to(self.userentry, userlabel,
                             Gtk.PositionType.RIGHT, 8, 4)
@@ -374,7 +388,7 @@ Useful for diagnosing connection problems''')
 
         # Load the default profile on startup
         self.load_settings()
-        self.profilename = 'defaults'
+        self.profilename = 'DEFAULT'
         # Set up Unity quicklist if we can support that
         if unity is True:
             self.create_unity_quicklist()
@@ -397,16 +411,16 @@ Useful for diagnosing connection problems''')
         height = int(round(cleangeo * mongeometry.height))
         return "%sx%s" % (width, height)
 
-    # Each section in the config file gets an entry in the profiles combobox
-    def populate_profiles_combobox(self):
-        self.profiles_combo.get_model().clear()
-        for profile in self.rd.list_profiles():
-            if profile != 'defaults':
-                self.profiles_combo.append_text(profile)
+    # Each section in the config file gets an entry in the host combobox
+    def populate_host_combobox(self):
+        self.host_combo_store.clear()
+        for profile in self.rd.saved_hosts:
+            if profile != 'DEFAULT':
+                self.host_combo_store.append([profile])
 
     # Each section in the config file gets an entry in the Unity quicklist
     def populate_unity_quicklist(self):
-        for profile in self.rd.list_profiles():
+        for profile in self.rd.saved_hosts:
             self.update_unity_quicklist(profile)
 
     # Create the Unity quicklist and populate it with our profiles
@@ -418,7 +432,7 @@ Useful for diagnosing connection problems''')
 
     # Append a new profile to the Unity quicklist
     def update_unity_quicklist(self, profile):
-        if profile != 'defaults':
+        if profile != 'DEFAULT':
             profile_menu_item = Dbusmenu.Menuitem.new()
             profile_menu_item.property_set(Dbusmenu.MENUITEM_PROP_LABEL,
                                            profile)
@@ -463,9 +477,12 @@ Useful for diagnosing connection problems''')
         self.connectbutton.show()
         error_text = WorkerThread.error_text
         return_code = WorkerThread.return_code
+        return_code_ignore = [62, 255]
         # return code 62 is a benign error code from rdesktop
         # return code 62 is not used by xfreerdp
-        if return_code is not 0 and return_code is not 62:
+        # return code 255 is a benign error code from xfreerdp
+        # return code 255 is not used by rdesktop
+        if return_code is not 0 and return_code not in return_code_ignore:
             # discard extra data from long error messages
             if len(error_text) > 300:
                 error_text = error_text[:300] + '...'
@@ -474,14 +491,25 @@ Useful for diagnosing connection problems''')
 
     # Triggered when the combobox is clicked.  We load the selected profile
     # from the config file.
-    def on_profiles_combo_changed(self, combo):
-        text = combo.get_active_text()
-        # Should we really iterate over the list of profiles here?
-        for profile in self.rd.list_profiles():
-            if text == profile:
+    def on_host_combo_changed(self, combo):
+        tree_iter = combo.get_active_iter()
+        if tree_iter != None:
+            model = combo.get_model()
+            name = model[tree_iter][0]
+            for profile in self.rd.saved_hosts:
+                if name == profile:
+                    self.rd.read_config(name)
+                    self.load_settings()
+            self.profilename = name
+        else:
+            entry = combo.get_child()
+            text = entry.get_text()
+            if text in self.rd.saved_hosts:
                 self.rd.read_config(text)
                 self.load_settings()
-        self.profilename = text
+            else:
+                self.rd.read_config('DEFAULT')
+                self.load_settings()
 
     # Triggered when the checkboxes are toggled
     def on_button_toggled(self, button, name):
@@ -505,13 +533,13 @@ Useful for diagnosing connection problems''')
         action_group.add_action(action_filemenu)
         # Why do the functions here execute on startup if we add parameters?
         action_group.add_actions([("SaveCurrentConfig", None,
-                                   "Save Current Profile", "<control>S", None,
+                                   "Save Host", "<control>S", None,
                                    self.save_current_config)])
         action_group.add_actions([("SaveCurrentConfigAsDefault", None,
-                                   "Save Current Profile as Default", "<control>D",
+                                   "Save Settings as Default", "<control>D",
                                    None, self.save_current_config_as_default)])
         action_group.add_actions([("DeleteCurrentConfig", None,
-                                   "Delete Current Profile", "<control><shift>S", None,
+                                   "Delete Host", "<control><shift>S", None,
                                    self.delete_current_config)])
         action_group.add_actions([("FileQuit", None,
                                    "Quit", "<control>Q", None,
@@ -538,44 +566,36 @@ Useful for diagnosing connection problems''')
 
     # When the save config button is clicked on the menu bar
     def save_current_config(self, widget):
-        if self.profilename == '' or self.profilename == 'defaults':
+        self.grab_textboxes()
+        self.profilename = self.rd.options['host']
+        if self.profilename == '' or self.profilename == 'DEFAULT':
             self.on_warn(None, 'No Profile Name',
                          'Please name your profile before saving.')
         else:
-            self.grab_textboxes()
             self.rd.save_config(self.profilename)
-            self.populate_profiles_combobox()
+            self.populate_host_combobox()
             if unity is True:
                 self.clean_unity_quicklist()
 
     # When the delete config button is clicked on the menu bar
     def delete_current_config(self, widget):
-        if self.profilename == '' or self.profilename == 'defaults':
+        if self.profilename == '' or self.profilename == 'DEFAULT':
             self.on_warn(None, 'Select a Profile',
                          'Please select a profile to delete.')
         else:
             self.rd.delete_config(self.profilename)
             # reload the default config
-            self.rd.read_config('defaults')
+            self.rd.read_config('DEFAULT')
             self.load_settings()
-            # Set profiles combobox to have no active item
-            self.profiles_combo.set_active(-1)
-            # Add a blank string to the head end of the combobox to 'clear' it
-            self.profiles_combo.prepend_text('')
-            # Set the blank string active to again, to 'clear' the combobox
-            self.profiles_combo.set_active(0)
-            # Now that we've 'cleared' the combobox text, let's delete the
-            # blank entry and then repopulate the entire combobox
-            active = self.profiles_combo.get_active()
-            self.profiles_combo.remove(active)
-            self.populate_profiles_combobox()
+            self.populate_host_combobox()
+            self.host_entry.set_text('')
             if unity is True:
                 self.clean_unity_quicklist()
 
     # When the save config button is clicked on the menu bar
     def save_current_config_as_default(self, widget):
         self.grab_textboxes()
-        self.rd.save_config('defaults')
+        self.rd.save_config('DEFAULT')
 
     # When the quit button is clicked on the menu bar
     def quit(self, widget):
@@ -597,7 +617,7 @@ Useful for diagnosing connection problems''')
 
     # Grab all textbox input
     def grab_textboxes(self):
-        self.rd.options['host'] = self.hostentry.get_text()
+        self.rd.options['host'] = self.host_entry.get_text()
         self.rd.options['user'] = self.userentry.get_text()
         self.rd.options['geometry'] = self.geometryentry.get_text()
         self.rd.options['clioptions'] = self.clioptionsentry.get_text()
@@ -616,7 +636,7 @@ Useful for diagnosing connection problems''')
     def on_about(self, widget):
         about = Gtk.AboutDialog()
         about.set_program_name("Rocket Depot")
-        about.set_version("0.22")
+        about.set_version("0.23")
         about.set_copyright("2014 David Roble")
         about.set_comments("rdesktop/xfreerdp Frontend")
         about.set_website("https://github.com/robled/rocket-depot")
@@ -626,7 +646,6 @@ Useful for diagnosing connection problems''')
 
     # Load all settings
     def load_settings(self):
-        self.hostentry.set_text(self.rd.options['host'])
         self.userentry.set_text(self.rd.options['user'])
         self.geometryentry.set_text(self.rd.options['geometry'])
         self.clioptionsentry.set_text(self.rd.options['clioptions'])
@@ -653,15 +672,12 @@ Useful for diagnosing connection problems''')
 
 
 def _main():
-    # Read the default profile and then save it if it doesn't already exist
     rocket_depot = RocketDepot()
     window = MainWindow(rocket_depot)
     window.connect("delete-event", Gtk.main_quit)
     window.show_all()
     # Hide the progress spinner until it is needed
     window.spinner.hide()
-    # Set focus to the host entry box on startup
-    window.hostentry.grab_focus()
     Gtk.main()
 
 
